@@ -1,6 +1,6 @@
 import numpy as np
 import pandas as pd
-from abc import ABC, abstractmethod
+from abc import abstractmethod, ABCMeta
 from typing import Optional
 from fastcore.all import store_attr
 
@@ -14,68 +14,7 @@ from ...utils.misc import to_np
 __all__ = ['EvalMetric']
 
 
-class OldEvalMetric(ABC):
-    r'''
-    Abstract class for evaluating performance of a model using some metric
-
-    Arguments:
-        targ_name: name of group in fold file containing regression targets
-        wgt_name: name of group in fold file containing datapoint weights
-    
-    .. Attention:: This class is depreciated in favour of :class:`~lumin.nn.metrics.eval_metric.EvalMetric`.
-        It is a copy of the old `EvalMetric` class used in lumin<=0.7.0.
-        It will be removed in V0.8
-    '''
-
-    # XXX remove in V0.8
-
-    def __init__(self, targ_name:str='targets', wgt_name:Optional[str]=None): self.targ_name,self.wgt_name,self.lower_metric_better = targ_name,wgt_name,True
-
-    def get_df(self, fy:FoldYielder, idx:int, y_pred:np.ndarray) -> pd.DataFrame:
-        r'''
-        Returns a DataFrame for the given fold containing targets, weights, and predictions
-
-        Arguments:
-            fy: :class:`~lumin.nn.data.fold_yielder.FoldYielder` interfacing to data
-            idx: fold index corresponding to fold for which y_pred was computed
-            y_pred: predictions for fold
-
-        Returns:
-            DataFrame for the given fold containing targets, weights, and predictions
-        '''
-
-        df = pd.DataFrame()
-        if self.wgt_name is not None: df['gen_weight'] = fy.get_column(column=self.wgt_name, n_folds=1, fold_idx=idx)
-        
-        targets = fy.get_column(column=self.targ_name, n_folds=1, fold_idx=idx)
-        if len(targets.shape) > 1:
-            for t in range(targets.shape[-1]): df[f'gen_target_{t}'] = targets[:,t]
-        else:
-            df['gen_target'] = targets
-        if len(y_pred.shape) > 1 and y_pred.shape[-1] > 1:
-            for p in range(y_pred.shape[-1]): df[f'pred_{p}'] = y_pred[:,p]
-        else:
-            df['pred'] = y_pred.squeeze()
-        return df
-
-    @abstractmethod
-    def evaluate(self, fy:FoldYielder, idx:int, y_pred:np.ndarray) -> float:
-        r'''
-        Evaluate the required metric for a given fold and set of predictions
-
-        Arguments:
-            fy: :class:`~lumin.nn.data.fold_yielder.FoldYielder` interfacing to data
-            idx: fold index corresponding to fold for which y_pred was computed
-            y_pred: predictions for fold
-
-        Returns:
-            metric value
-        '''
-
-        pass
-
-
-class EvalMetric(Callback):
+class EvalMetric(Callback, metaclass=ABCMeta):
     r'''
     Abstract class for evaluating performance of a model using some metric
 
@@ -172,10 +111,9 @@ class EvalMetric(Callback):
 
         self.model = model
         preds = self.model.predict(inputs, bs=bs)
-        return self.evaluate_preds(fy=fy, fold_idx=fold_idx, preds=preds, targets=targets, weights=weights, bs=bs)
+        return self.evaluate_preds(fy=fy, fold_idx=fold_idx, preds=preds, targets=targets, weights=weights)
 
-    def evaluate_preds(self, fy:FoldYielder, fold_idx:int, preds:np.ndarray, targets:np.ndarray, weights:Optional[np.ndarray]=None,
-                       bs:Optional[int]=None) -> float:
+    def evaluate_preds(self, fy:FoldYielder, fold_idx:int, preds:np.ndarray, targets:np.ndarray, weights:Optional[np.ndarray]=None) -> float:
         r'''
         Computes metric value from predictions. fy and fold_idx arguments necessary in case the metric requires extra information beyond inputs, 
         tragets, and weights.
