@@ -1,12 +1,11 @@
 import numpy as np
 import pandas as pd
 from typing import List, Optional, Union, Generator, Callable
-from abc import ABC, abstractmethod
+from abc import ABCMeta, abstractmethod
 
-from torch.tensor import Tensor
+from torch import Tensor
 from torch import optim
 
-from ..data.batch_yielder import BatchYielder
 from ..data.fold_yielder import FoldYielder
 from ..callbacks.abs_callback import AbsCallback
 
@@ -19,28 +18,7 @@ class FitParams():
         self.epoch,self.sub_epoch = 0,0
 
 
-class OldAbsModel(ABC):
-    r'''
-    .. Attention:: This class is depreciated in favour of :class:`~lumin.nn.models.abs_model.AbsModel`.
-        It is a copy of the old `AbsModel` class used in lumin<=0.6.
-        It will be removed in V0.8
-    '''
-
-    # XXX remove in V0.8
-
-    def __init__(self): pass
-    
-    @abstractmethod
-    def fit(self, batch_yielder:BatchYielder, callbacks:List[AbsCallback]) -> float: pass
-    
-    @abstractmethod
-    def evaluate(self, inputs:Tensor, targets:Tensor, weights:Optional[Tensor]) -> float: pass
-
-    @abstractmethod   
-    def predict(self, inputs, as_np:bool=True) -> Union[np.ndarray, Tensor]: pass
-
-
-class AbsModel(ABC):
+class AbsModel(metaclass=ABCMeta):
     '''Abstract model class for typing'''
     def __init__(self): pass
     
