@@ -1,4 +1,4 @@
-from typing import Tuple, Dict, List, Optional, Union, Callable, Generator
+from typing import Tuple, Dict, List, Optional, Union
 from fastprogress import master_bar, progress_bar
 import numpy as np
 from collections import OrderedDict
@@ -9,16 +9,12 @@ from fastcore.all import is_listy
 from sklearn.ensemble import RandomForestRegressor, RandomForestClassifier
 
 from ..nn.data.fold_yielder import FoldYielder
-from ..nn.data.batch_yielder import BatchYielder
 from ..nn.models.model_builder import ModelBuilder
-from ..nn.models.model import Model, OldModel
-from ..nn.callbacks.opt_callbacks import LRFinder, OldLRFinder
-from ..nn.callbacks.cyclic_callbacks import OldAbsCyclicCallback
-from ..nn.callbacks.model_callbacks import OldAbsModelCallback
+from ..nn.models.model import Model
+from ..nn.callbacks.opt_callbacks import LRFinder
 from ..plotting.training import plot_lr_finders
 from ..plotting.plot_settings import PlotSettings
 
-from torch import optim, Tensor
 
 import matplotlib.pyplot as plt
 
@@ -78,49 +74,6 @@ def get_opt_rf_params(x_trn:np.ndarray, y_trn:np.ndarray, x_val:np.ndarray, y_va
     return best_params, best_m
 
 
-def fold_lr_find(fy:FoldYielder, model_builder:ModelBuilder, bs:int,
-                 train_on_weights:bool=True, shuffle_fold:bool=True, n_folds:int=-1, lr_bounds:Tuple[float,float]=[1e-5, 10],
-                 callback_partials:Optional[List[partial]]=None, plot_settings:PlotSettings=PlotSettings(),
-                 bulk_move:bool=True, plot_savename:Optional[str]=None) -> List[OldLRFinder]:
-    r'''
-    .. Attention:: This class is depreciated in favour of :meth:`~lumin.optimisation.hyper_param.lr_find`.
-        It will be removed in V0.8
-    '''
-
-    # XXX remove in V0.8
-
-    if callback_partials is None: callback_partials = []
-    idxs = range(fy.n_folds) if n_folds < 1 else range(min(n_folds, fy.n_folds))
-    lr_finders = []
-    tmr = timeit.default_timer()
-    nb = None
-    for trn_id in progress_bar(idxs):
-        model = OldModel(model_builder)
-        trn_fold = fy.get_fold(trn_id)
-        if nb is None: nb = len(trn_fold['targets'])//bs
-        lr_finder = OldLRFinder(nb=nb, lr_bounds=lr_bounds, model=model)
-        cyclic_callback,callbacks = None,[]
-        for c in callback_partials: callbacks.append(c(model=model))
-        for c in callbacks:
-            if isinstance(c, OldAbsCyclicCallback): c.set_nb(nb)
-        for c in callbacks:
-            if isinstance(c, OldAbsModelCallback): c.set_cyclic_callback(cyclic_callback)
-        for c in callbacks:
-            c.on_train_begin()
-        lr_finder.on_train_begin()
-        batch_yielder = BatchYielder(**trn_fold, objective=model_builder.objective, bs=bs, use_weights=train_on_weights, shuffle=shuffle_fold,
-                                     bulk_move=bulk_move)
-        model.fit(batch_yielder, callbacks+[lr_finder])
-        lr_finders.append(lr_finder)
-    del batch_yielder
-    del model
-        
-    print("LR finder took {:.3f}s ".format(timeit.default_timer()-tmr))
-    plot_lr_finders(lr_finders, loss_range='auto', settings=plot_settings, log_y='auto' if 'regress' in model_builder.objective.lower() else False,
-                    savename=plot_savename)
-    return lr_finders
-
-
 def lr_find(fy:FoldYielder, model_builder:ModelBuilder, bs:int, n_epochs:int=1,
             train_on_weights:bool=True, n_folds:int=-1, lr_bounds:Tuple[float,float]=[1e-5, 10],
             cb_partials:Optional[List[partial]]=None, plot_settings:PlotSettings=PlotSettings(),
