@@ -1,17 +1,18 @@
 import numpy as np
-from typing import Union, List, Tuple, Optional, Any
+from typing import Callable, Union, List, Tuple, Optional, Any
 import pandas as pd
 import sympy
 from functools import partial
-from fastcore.all import partialler
+import types
+import inspect
 
 from sklearn.utils import resample
 
-from torch.tensor import Tensor
+from torch import Tensor
 import torch
 import torch.nn as nn
 
-__all__ = ['to_np', 'to_device', 'to_tensor', 'str2bool', 'to_binary_class', 'ids2unique', 'FowardHook', 'subsample_df', 'is_partially']
+__all__ = ['to_np', 'to_device', 'to_tensor', 'str2bool', 'to_binary_class', 'ids2unique', 'ForwardHook', 'BackwardHook', 'subsample_df', 'is_partially']
 
 device = torch.device('cuda') if torch.cuda.is_available() else torch.device('cpu')  # TODO: make device choosable by user
 
@@ -115,7 +116,7 @@ def ids2unique(ids: Union[List[int], np.ndarray]) -> np.ndarray:
     return (primes**ids).prod(axis=-1)
 
 
-class FowardHook():
+class ForwardHook():
     r'''
     Create a hook for performing an action based on the forward pass thorugh a nn.Module
 
@@ -128,7 +129,7 @@ class FowardHook():
         >>> model.predict(inputs)
         >>> print(hook.inputs)
     '''
-    def __init__(self, module:nn.Module, hook_fn:Optional=None):
+    def __init__(self, module:nn.Module, hook_fn:Optional[Callable[[nn.Module, Union[Tensor,Tuple[Tensor]], Union[Tensor,Tuple[Tensor]]],None]]=None):
         self.input,self.output = None,None
         if hook_fn is not None: self.hook_fn = partial(hook_fn, self)
         self.hook = module.register_forward_hook(self.hook_fn)
@@ -155,6 +156,25 @@ class FowardHook():
         self.hook.remove()
 
 
+class BackwardHook(ForwardHook):
+    r'''
+    Create a hook for performing an action based on the backward pass thorugh a nn.Module
+
+    Arguments:
+        module: nn.Module to hook
+        hook_fn: Optional function to perform. Default is to record input and output of module
+
+    Examples::
+        >>> hook = BackwardHook(model.tail.dense)
+        >>> model.predict(inputs)
+        >>> print(hook.inputs)
+    '''
+    def __init__(self, module:nn.Module, hook_fn:Optional[Callable[[nn.Module, Union[Tensor,Tuple[Tensor]], Union[Tensor,Tuple[Tensor]]],None]]=None):
+        self.input,self.output = None,None
+        if hook_fn is not None: self.hook_fn = partial(hook_fn, self)
+        self.hook = module.register_backward_hook(self.hook_fn)
+
+
 def subsample_df(df:pd.DataFrame, objective:str, targ_name:str, n_samples:Optional[int]=None, replace:bool=False, strat_key:Optional[str]=None,
                  wgt_name:Optional[str]=None) -> pd.DataFrame:
     r'''
@@ -185,7 +205,7 @@ def subsample_df(df:pd.DataFrame, objective:str, targ_name:str, n_samples:Option
 
 def is_partially(var:Any) -> bool:
     r'''
-    Retuns true if var is partial or partialler, else false.
+    Retuns true if var is partial, function, or class, else false.
 
     Arguments:
         var: variable to inspect
@@ -194,4 +214,5 @@ def is_partially(var:Any) -> bool:
         true if var is partial or partialler, else false
     '''
 
-    return isinstance(var, partial) or isinstance(var, partialler)
+    return isinstance(var, (partial,types.FunctionType)) or inspect.isclass(var)
+
